@@ -57,15 +57,12 @@ def writeVarToDisk():
     vars.writeCeciliaPrefsToFile()
 
 ###### Start / Stop / Drivers ######
-def startCeciliaSound(timer=True):
+def startCeciliaSound(timer=True, rec=False):
     # Check if soundfile is loaded
-    # if 0: # no sound...
-    #     showErrorDialog('No input sound file!', 'Please load one...')
-    #     for key in getVar("userInputs").keys():
-    #         if not os.path.isfile(getVar("userInputs")[key]['path']):
-    #             getControlPanel().getCfileinFromName(key).onLoadFile()
-    #     stopCeciliaSound()
-    #     return
+    for key in getVar("userInputs").keys():
+        if not os.path.isfile(getVar("userInputs")[key]['path']):
+            showErrorDialog('"%s", no input sound file!' % getControlPanel().getCfileinFromName(key).label, 'Please load one...')
+            getControlPanel().getCfileinFromName(key).onLoadFile()
     getControlPanel().resetMeter()
     getVar("audioServer").shutdown()
     getVar("audioServer").reinit()
@@ -76,29 +73,27 @@ def startCeciliaSound(timer=True):
         showErrorDialog("Wow...!", "No module to load.")
     getVar("grapher").toolbar.convertSlider.Hide()
     getVar("presetPanel").presetChoice.setEnable(False)
-    getControlPanel().durationSlider.setEnable(False)
-    getVar("audioServer").start(timer=timer)
+    getVar("audioServer").start(timer=timer, rec=rec)
     getVar("grapher").toolbar.loadingMsg.SetForegroundColour(TITLE_BACK_COLOUR)
+    wx.CallAfter(getVar("grapher").toolbar.loadingMsg.Refresh)
 
 def stopCeciliaSound():
-    getVar("audioServer").stop()
-    if getVar("currentModule") != None:
-        getVar("audioServer").checkForAutomation()
-        getVar("currentModule")._checkForAutomation()
-        getVar("grapher").checkForAutomation()
-    time.sleep(.25)
-    if getVar('interface') != None:
-        getControlPanel().transportButtons.setPlay(False)
-        getControlPanel().transportButtons.setRecord(False)
-        getVar("presetPanel").presetChoice.setEnable(True)
-        getControlPanel().durationSlider.setEnable(True)
-        wx.CallAfter(getControlPanel().vuMeter.reset)
+    if getVar("audioServer").isAudioServerRunning():
+        getVar("audioServer").stop()
+        if getVar("currentModule") != None:
+            getVar("audioServer").checkForAutomation()
+            getVar("currentModule")._checkForAutomation()
+            getVar("grapher").checkForAutomation()
+        time.sleep(.25)
+    resetControls()
 
 def resetControls():
     if getVar('interface') != None:
         getControlPanel().transportButtons.setPlay(False)
         getControlPanel().transportButtons.setRecord(False)
         getVar("presetPanel").presetChoice.setEnable(True)
+        if getControlPanel().tmpTotalTime != getVar("totalTime"):
+            getControlPanel().setTotalTime(getControlPanel().tmpTotalTime, True)
         wx.CallAfter(getControlPanel().vuMeter.reset)
     
 def audioServerIsRunning(state):
@@ -107,23 +102,21 @@ def audioServerIsRunning(state):
             getControlPanel().transportButtons.setPlay(True)
 
 def queryAudioMidiDrivers():
-    inputs, selectedInput, outputs, selectedOutput, midiInputs, selectedMidiInput = getVar("audioServer").getAvailableAudioMidiDrivers()
+    inputs, inputIndexes, defaultInput, outputs, outputIndexes, defaultOutput, midiInputs, midiInputIndexes, defaultMidiInput = getVar("audioServer").getAvailableAudioMidiDrivers()
 
     setVar("availableAudioOutputs",  outputs)
-    if getVar("audioOutput") < len(outputs):
-        setVar("audioOutput", getVar("audioOutput"))
-    else:
+    setVar("availableAudioOutputIndexes",  outputIndexes)
+    if getVar("audioOutput") not in outputIndexes:
         try:
-            setVar("audioOutput", outputs.index(selectedOutput))
+            setVar("audioOutput", outputIndexes[outputs.index(defaultOutput)])
         except:
             setVar("audioOutput", 0)
 
     setVar("availableAudioInputs", inputs)
-    if getVar("audioInput") < len(inputs):
-        setVar("audioInput", getVar("audioInput"))
-    else:
+    setVar("availableAudioInputIndexes", inputIndexes)
+    if getVar("audioInput") not in inputIndexes:
         try:
-            setVar("audioInput", inputs.index(selectedInput))
+            setVar("audioInput", inputIndexes[inputs.index(defaultInput)])
         except:
             setVar("audioInput", 0)
 
@@ -132,11 +125,10 @@ def queryAudioMidiDrivers():
     else:
         setVar("useMidi", 1)
     setVar("availableMidiInputs", midiInputs)
-    if getVar("midiDeviceIn") <= len(midiInputs):
-        setVar("midiDeviceIn", getVar("midiDeviceIn"))
-    else:
+    setVar("availableMidiInputIndexes", midiInputIndexes)
+    if getVar("midiDeviceIn") not in midiInputIndexes:
         try:
-            setVar("midiDeviceIn", midiInputs.index(selectedMidiInput))
+            setVar("midiDeviceIn", midiInputIndexes[midiInputs.index(defaultMidiInput)])
         except:
             setVar("midiDeviceIn", 0)
 
@@ -207,15 +199,15 @@ def showErrorDialog(title, msg):
 def loadPlayerEditor(app_type):
     if getVar("systemPlatform")  == 'win32':
         wildcard =  "Executable files (*.exe)|*.exe|"     \
-                    "All files (*.*)|*.*"
+                    "All files|*"
     elif getVar("systemPlatform")  == 'darwin':
         wildcard =  "Application files (*.app)|*.app|"     \
-                    "All files (*.*)|*.*"
+                    "All files|*"
     else:
-        wildcard = "All files (*.*)|*.*"
+        wildcard = "All files|*"
     
     path = ''
-    dlg = wx.FileDialog(None, message="Choose a soundfile %s..." % app_type,
+    dlg = wx.FileDialog(None, message="Choose a %s..." % app_type,
                              defaultDir=os.path.expanduser('~'),
                              wildcard=wildcard, style=wx.OPEN)
 
@@ -224,15 +216,17 @@ def loadPlayerEditor(app_type):
     dlg.Destroy()
 
     if path:
-        if app_type == 'player':
+        if app_type == 'soundfile player':
             setVar("soundfilePlayer", path)
-        elif app_type == 'editor':
+        elif app_type == 'soundfile editor':
             setVar("soundfileEditor", path)
+        elif app_type == 'text editor':
+            setVar("textEditor", path)
 
 def listenSoundfile(soundfile):
     if getVar("soundfilePlayer") == '':
         showErrorDialog("Preferences not set", "Choose a soundfile player first.")
-        loadPlayerEditor('player')
+        loadPlayerEditor('soundfile player')
     if os.path.isfile(soundfile):
         app = getVar("soundfilePlayer")
         #app = slashifyText(getVar("soundfilePlayer"))
@@ -244,18 +238,18 @@ def listenSoundfile(soundfile):
             try:
                 Popen([app, soundfile], shell=False)
             except OSError, OSError2:
-                print 'Unable to open desired software:\t' + app
+                print 'Unable to open desired software:\n' + app
         else:
             cmd = '"%s" "%s"' % (app, soundfile)
             try:
                 Popen(cmd, shell=True)
             except OSError, OSError2:
-                print 'Unable to open desired software:\t' + app
+                print 'Unable to open desired software:\n' + app
 
 def editSoundfile(soundfile):
     if getVar("soundfileEditor") == '':
         showErrorDialog("Preferences not set", "Choose a soundfile editor first.")
-        loadPlayerEditor('editor')
+        loadPlayerEditor('soundfile editor')
     if os.path.isfile(soundfile):
         app = getVar("soundfileEditor")
         #app = slashifyText(getVar("soundfileEditor"))
@@ -267,13 +261,34 @@ def editSoundfile(soundfile):
             try:
                 Popen([app, soundfile], shell=False)
             except OSError, OSError2:
-                print 'Unable to open desired software:\t' + app
+                print 'Unable to open desired software:\n' + app
         else:
             cmd = '%s %s' % (app, soundfile)
             try:
                 Popen(cmd, shell=True)
             except OSError, OSError2:
-                print 'Unable to open desired software:\t' + app
+                print 'Unable to open desired software:\n' + app
+
+def openCurrentFileAsText(curfile):
+    if getVar("textEditor") == '':
+        showErrorDialog("Preferences not set", "Choose a text editor first.")
+        loadPlayerEditor('text editor')
+    if os.path.isfile(curfile):
+        app = getVar("textEditor")
+        if getVar("systemPlatform")  == 'darwin':
+            cmd = 'open -a "%s" "%s"' % (app, os.path.join(os.getcwd(), curfile))
+            Popen(cmd, shell=True, cwd=os.path.expanduser("~"))
+        elif getVar("systemPlatform")  == 'win32':
+            try:
+                Popen([app, curfile], shell=False)
+            except OSError, OSError2:
+                print 'Unable to open desired software:\n' + app
+        else:
+            cmd = '%s %s' % (app, curfile)
+            try:
+                Popen(cmd, shell=True)
+            except OSError, OSError2:
+                print 'Unable to open desired software:\n' + app
 
 ###### Preset functions ######
 def deletePreset(preset):
