@@ -85,45 +85,228 @@ class CECControl(scrolled.ScrolledPanel):
         self.peak = ''
         self.time = self.nonZeroTime = 0
         self.charNumForLabel = 34
+        self.bounce_dlg = None
+        self.tmpTotalTime = CeciliaLib.getVar("totalTime")
 
-        sizerMain = wx.FlexGridSizer(3,1)
-        
-        sizerMain.Add(Separator(self, (230,1), colour=TITLE_BACK_COLOUR), 1, wx.EXPAND)
-                
-        ##### Control Panel #####
+        self.sizerMain = wx.FlexGridSizer(3,1)
+
+        self.sizerMain.Add(Separator(self, (230,1), colour=TITLE_BACK_COLOUR), 1, wx.EXPAND)
+
+        ##### Transport Panel #####
         controlPanel = wx.Panel(self, -1, style=wx.NO_BORDER)
         controlPanel.SetBackgroundColour(TITLE_BACK_COLOUR)
         controlSizer = wx.FlexGridSizer(1,4)        
         self.transportButtons = Transport(controlPanel, outPlayFunction=self.onPlayStop, outRecordFunction=self.onRec,
-                                                  backgroundColour=TITLE_BACK_COLOUR, borderColour=WIDGET_BORDER_COLOUR)        
-        self.clocker = Clocker(controlPanel, backgroundColour=TITLE_BACK_COLOUR, borderColour=WIDGET_BORDER_COLOUR)        
+                                                  backgroundColour=TITLE_BACK_COLOUR, borderColour=WIDGET_BORDER_COLOUR)
+        self.clocker = Clocker(controlPanel, backgroundColour=TITLE_BACK_COLOUR, borderColour=WIDGET_BORDER_COLOUR)
         controlSizer.Add(self.transportButtons, 0, wx.ALIGN_LEFT | wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
-        fakePanel = wx.Panel(controlPanel, -1, size=(10, self.GetSize()[1]))
+        fakePanel = wx.Panel(controlPanel, -1, size=(35, self.GetSize()[1]))
         fakePanel.SetBackgroundColour(TITLE_BACK_COLOUR)
         controlSizer.Add(fakePanel)
-        controlSizer.Add(self.clocker, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
+        controlSizer.Add(self.clocker, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
         ffakePanel = wx.Panel(controlPanel, -1, size=(5, self.GetSize()[1]))
         ffakePanel.SetBackgroundColour(TITLE_BACK_COLOUR)
         controlSizer.Add(ffakePanel)
-        controlSizer.AddGrowableCol(1)        
-        controlPanel.SetSizer(controlSizer)        
-        sizerMain.Add(controlPanel, 1, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, 0)
-        
-        sizerMain.Add(Separator(self, (230,1), colour=TITLE_BACK_COLOUR), 1, wx.EXPAND)        
-        sizerMain.Add(Separator(self, (230,2), colour=BORDER_COLOUR), 1, wx.EXPAND)
-        sizerMain.AddSpacer((5,1))
-        
+        #controlSizer.AddGrowableCol(1)
+        controlPanel.SetSizer(controlSizer)
+        self.sizerMain.Add(controlPanel, 1, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.ALL, 0)
+
+        self.sizerMain.Add(Separator(self, (230,1), colour=TITLE_BACK_COLOUR), 1, wx.EXPAND)
+        self.sizerMain.Add(Separator(self, (230,2), colour=BORDER_COLOUR), 1, wx.EXPAND)
+        self.sizerMain.AddSpacer((5,1))
+
+        self.tabs = TabsPanel(self, outFunction=self.onTogglePanels)
+        self.sizerMain.Add(self.tabs, 1, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, 0)
+
         ##### Input Panel #####
-        inputPanel, isEmpty = self.createInputPanel(self)     
-        sizerMain.Add(inputPanel, 1, wx.EXPAND | wx.ALL, 0)
-        
+        self.inOutSeparators = []
+        isEmpty = self.createInputPanel()
+        self.sizerMain.Add(self.inputPanel, 1, wx.EXPAND | wx.ALL, 0)
         if not isEmpty:
-            sizerMain.AddSpacer((5,2))
-            sizerMain.Add(Separator(self, (230,2), colour=BORDER_COLOUR), 1, wx.EXPAND)
-            sizerMain.AddSpacer((5,1))
+            sep = Separator(self, (230,2), colour=BACKGROUND_COLOUR)
+            self.sizerMain.Add(sep, 1, wx.EXPAND)
+            self.inOutSeparators.append(sep)
+            sep = Separator(self, (230,2), colour=BORDER_COLOUR)
+            self.sizerMain.Add(sep, 1, wx.EXPAND)
+            self.inOutSeparators.append(sep)
+            sep = Separator(self, (230,1), colour=BACKGROUND_COLOUR)
+            self.sizerMain.Add(sep, 1, wx.EXPAND)
+            self.inOutSeparators.append(sep)
 
-       
         ###### Output Panel #####
+        self.createOutputPanel()
+        self.sizerMain.Add(self.outputPanel, 1, wx.EXPAND | wx.ALL, 0)
+        sep = Separator(self, (230,2), colour=BACKGROUND_COLOUR)
+        self.sizerMain.Add(sep, 1, wx.EXPAND)
+        self.inOutSeparators.append(sep)
+        sep = Separator(self, (230,2), colour=BORDER_COLOUR)
+        self.sizerMain.Add(sep, 1, wx.EXPAND)
+        self.inOutSeparators.append(sep)
+        sep = Separator(self, (230,1), colour=BACKGROUND_COLOUR)
+        self.sizerMain.Add(sep, 1, wx.EXPAND)
+        self.inOutSeparators.append(sep)
+
+        ### Plugins panel ###
+        self.createPluginPanel()
+        self.sizerMain.Add(self.pluginsPanel, 1, wx.EXPAND | wx.ALL, 0)
+        self.sizerMain.Show(self.pluginsPanel, False)
+ 
+        self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        controlPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        self.inputPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        self.outputPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        self.peakLabel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        self.durationSlider.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        self.gainSlider.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        self.vuMeter.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        self.pluginsPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+
+        self.SetSizer(self.sizerMain)
+
+        self.SetAutoLayout(1)
+        self.SetupScrolling(scroll_x = False)
+
+        wx.CallAfter(self.updateOutputFormat)
+
+    def OnLooseFocus(self, event):
+        win = wx.FindWindowAtPointer()
+        if win != None:
+            win = win.GetTopLevelParent()
+            if win not in [CeciliaLib.getVar("mainFrame"), CeciliaLib.getVar("interface")]:
+                win.Raise()
+        event.Skip()
+
+    def onTogglePanels(self, state):
+        if state == 0:
+            self.sizerMain.Show(self.pluginsPanel, False, True)
+            self.sizerMain.Show(self.inputPanel, True, True)
+            self.sizerMain.Show(self.outputPanel, True, True)
+            [self.sizerMain.Show(sep, True, True) for sep in self.inOutSeparators]
+        else:
+            self.sizerMain.Show(self.pluginsPanel, True, True)
+            self.sizerMain.Show(self.inputPanel, False, True)
+            self.sizerMain.Show(self.outputPanel, False, True)
+            [self.sizerMain.Show(sep, False, True) for sep in self.inOutSeparators]
+        self.sizerMain.Layout()
+
+    def createGrapherLines(self, plugin):
+        knobs = [plugin.knob1, plugin.knob2, plugin.knob3]
+        grapher = CeciliaLib.getVar("grapher")
+        choice = grapher.toolbar.getPopupChoice()
+        choice.extend([knob.getLongLabel() for knob in knobs])
+        grapher.toolbar.setPopupChoice(choice)
+        for knob in knobs:
+            func = '0 %f 1 %f' % (knob.GetValue(), knob.GetValue())
+            func = [float(v.replace('"', '')) for v in func.split()]
+            func = [[func[i*2] * CeciliaLib.getVar("totalTime"), func[i*2+1]] for i in range(len(func) / 2)]
+            mini = knob.getRange()[0]
+            maxi = knob.getRange()[1]
+            colour = chooseColourFromName('red')
+            label = knob.getLongLabel()
+            log = knob.getLog()
+            name = knob.getName()
+            grapher.plotter.createLine(func, (mini, maxi), colour, label, log, name, 8192, knob, '')
+            grapher.plotter.getData()[-1].setShow(0)
+            grapher.plotter.draw()
+
+    def removeGrapherLines(self, plugin):
+        knobs = [plugin.knob1, plugin.knob2, plugin.knob3]
+        tmp = [knob.getLongLabel() for knob in knobs]
+        names = [knob.getName() for knob in knobs]
+        grapher = CeciliaLib.getVar("grapher")
+        choice = grapher.toolbar.getPopupChoice()
+        for label in tmp:
+            if label in choice:
+                choice.remove(label)
+        grapher.toolbar.setPopupChoice(choice)
+        grapher.plotter.removeLines(names)
+
+    def replacePlugin(self, order, new):
+        self.pluginsParams[order][self.oldPlugins[order]] = self.plugins[order].getParams()
+        oldPlugin = self.plugins[order]
+        if self.oldPlugins[order] != 0:
+            self.removeGrapherLines(oldPlugin)
+        plugin = self.pluginsDict[new](self.pluginsPanel, self.replacePlugin, order)
+
+        if new != 'None':    
+            CeciliaLib.setPlugins(plugin, order)
+            self.createGrapherLines(plugin)
+        else:
+            CeciliaLib.setPlugins(None, order)
+
+        ind = PLUGINS_CHOICE.index(plugin.getName())
+        self.oldPlugins[order] = ind
+        plugin.setParams(self.pluginsParams[order][ind])
+        if CeciliaLib.getVar("systemPlatform")  == 'darwin':
+            self.pluginSizer.Replace(oldPlugin, plugin)
+        else:
+            item = self.pluginSizer.GetItem(oldPlugin)
+            item.DeleteWindows()
+            self.pluginSizer.Insert([2,8,13][order], plugin, 0) 
+        self.plugins[order] = plugin       
+        self.pluginsPanel.Layout()
+        if CeciliaLib.getVar("audioServer").isAudioServerRunning():
+            CeciliaLib.getVar("audioServer").setPlugin(order)
+
+    def setPlugins(self, pluginsDict):
+        for key in pluginsDict.keys():
+            self.replacePlugin(key, pluginsDict[key][0])
+            self.plugins[key].setParams(pluginsDict[key][1])
+            self.plugins[key].setStates(pluginsDict[key][2])
+
+    def updateTime(self, time):
+        self.setTime(time)
+        self.GetParent().grapher.plotter.drawCursor(time)
+
+    def updateAmps(self, amps):
+        self.vuMeter.setAmplitude(amps)
+
+    def createInputPanel(self):
+        isEmpty = True
+        self.inputPanel = wx.Panel(self, -1, style=wx.NO_BORDER)
+        inputSizer = wx.FlexGridSizer(5,1)
+        
+        self.cfileinList = []
+        samplersList = []
+        widgets = CeciliaLib.getVar("interfaceWidgets")
+        
+        for w in range(len(widgets)):
+            if widgets[w]['type'] == 'cfilein':
+                cFileIn = Cfilein(self.inputPanel, label=widgets[w].get('label', ''), name=widgets[w]['name'])
+                self.cfileinList.append(cFileIn)
+            elif widgets[w]['type'] == 'csampler':
+                cSampler = CSampler(self.inputPanel, label=widgets[w].get('label', ''), name=widgets[w]['name'])
+                self.cfileinList.append(cSampler)
+                samplersList.append(cSampler)
+
+        CeciliaLib.setVar("userSamplers", samplersList)
+
+        if self.cfileinList != []:
+            isEmpty = False
+            # Section title
+            inputTextPanel = wx.Panel(self.inputPanel, -1, style=wx.NO_BORDER)
+            inputTextPanel.SetBackgroundColour(TITLE_BACK_COLOUR)
+            inputTextSizer = wx.FlexGridSizer(1,1)
+            inputText = wx.StaticText(inputTextPanel, -1, 'INPUT ')
+            inputText.SetFont(wx.Font(SECTION_TITLE_FONT, wx.NORMAL, wx.NORMAL, wx.BOLD, face=FONT_FACE))
+            inputText.SetBackgroundColour(TITLE_BACK_COLOUR)
+            inputText.SetForegroundColour(SECTION_TITLE_COLOUR)
+            inputTextSizer.Add(inputText, 0, wx.ALIGN_RIGHT | wx.ALL, 3)
+            inputTextSizer.AddGrowableCol(0)
+            inputTextPanel.SetSizer(inputTextSizer)
+            inputSizer.Add(inputTextPanel, 1, wx.EXPAND| wx.ALIGN_RIGHT | wx.ALL, 0)
+        
+        for i in range(len(self.cfileinList)):
+            inputSizer.Add(self.cfileinList[i], 1, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, -1)
+            if i != len(self.cfileinList)-1:
+                inputSizer.Add(Separator(self.inputPanel, size=(230,1)), 1, wx.EXPAND)
+        
+        inputSizer.AddGrowableCol(0)
+        self.inputPanel.SetSizer(inputSizer)
+        
+        return isEmpty
+
+    def createOutputPanel(self):
         self.outputPanel = wx.Panel(self, -1, style=wx.NO_BORDER)
         self.outputPanel.SetBackgroundColour(BACKGROUND_COLOUR)
         outputSizer = wx.FlexGridSizer(5,1)
@@ -149,13 +332,12 @@ class CECControl(scrolled.ScrolledPanel):
                                         colour=CONTROLLABEL_BACK_COLOUR, outFunction=self.onSelectOutputFilename)
         self.filenameLabel.SetToolTip(CECTooltip(TT_OUTPUT))
         self.filenameLabel.setItalicLabel('File name')
-        outLine1.Add(self.filenameLabel, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL , 0)
+        outLine1.Add(self.filenameLabel, 0, wx.RIGHT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL , 20)
         
         outLine1.AddSpacer((8,1))
  
-        outToolbox = ToolBox(self.outputPanel, tools=['play','edit','load','recycle'],
-                            outFunction=[self.listenSoundfile, self.editSoundfile,
-                                        self.onSelectOutputFilename, self.onReuseOutputFile])
+        outToolbox = ToolBox(self.outputPanel, tools=['play','edit','recycle'],
+                            outFunction=[self.listenSoundfile, self.editSoundfile, self.onReuseOutputFile])
         outLine1.Add(outToolbox, 0,  wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL | wx.RIGHT | wx.TOP, 2)
         
         outputSizer.Add(outLine1, 1, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.BOTTOM, 7)
@@ -169,7 +351,7 @@ class CECControl(scrolled.ScrolledPanel):
         # Duration Slider
         outputSizer.AddSpacer((3,1))
         self.durationSlider = ControlSlider(self.outputPanel,
-                                                    0.001, 3600, CeciliaLib.getVar("defaultTotalTime"),
+                                                    0.01, 3600, CeciliaLib.getVar("defaultTotalTime"),
                                                     size=(220,15), log=True, outFunction=self.setTotalTime)
         self.durationSlider.setSliderHeight(10)
         self.durationSlider.SetToolTip(CECTooltip(TT_DUR_SLIDER))
@@ -230,30 +412,28 @@ class CECControl(scrolled.ScrolledPanel):
         peakSizer.Add(self.peakLabel, 0, wx.ALIGN_LEFT | wx.TOP, 1)
         self.lineSizer.Add(peakSizer, 0, wx.ALIGN_LEFT | wx.LEFT, 10)
 
+        outputTextPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        outToolbox.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+
         outputSizer.Add(self.meterSizer, 1, wx.EXPAND)
         outputSizer.Add(self.lineSizer, 0, wx.ALIGN_LEFT | wx.LEFT | wx.BOTTOM, 7)
         outputSizer.AddGrowableRow(9)
         self.outputPanel.SetSizer(outputSizer)
-        sizerMain.Add(self.outputPanel, 1, wx.EXPAND | wx.ALL, 0)
 
-        sizerMain.AddSpacer((5,2))
-        sizerMain.Add(Separator(self, (230,2), colour=BORDER_COLOUR), 1, wx.EXPAND)
-        sizerMain.AddSpacer((5,1))
-
-        ### Plugins panel ###
+    def createPluginPanel(self):
         self.oldPlugins = [0,0,0]
         for i in range(3):
             CeciliaLib.setPlugins(None, i)
-        self.pluginsParams = {  0: [[0,0,0,0], [.25,1,.5,1], [1,1000,1,1], [.5,.2,.5,1], [1000,1,-3,1], 
+        self.pluginsParams = {  0: [[0,0,0,0], [.25,1,.5,1], [.25,.7,5000,1], [1,1000,1,1], [.5,.2,.5,1], [1000,1,-3,1], 
                                 [0,0,0,1], [-20,3,0,1], [-70,0.005,.01,1], [.7,.7,-12,1], [8,1,0,1], [100,5,1.1,1],
                                 [.1,0,0.5,1], [0.5,0.25,0.25,1], [-7,0,0.5,1], [80,2.01,0.33,1], [80,0.5,0.33,1]],
-                                1: [[0,0,0,0], [.25,1,.5,1], [1,1000,1,1], [.5,.2,.5,1], [1000,1,-3,1], 
+                                1: [[0,0,0,0], [.25,1,.5,1], [.25,.7,5000,1], [1,1000,1,1], [.5,.2,.5,1], [1000,1,-3,1], 
                                 [0,0,0,1], [-20,3,0,1], [-70,0.005,.01,1], [.7,.7,-12,1], [8,1,0,1], [100,5,1.1,1],
                                 [.1,0,0.5,1], [0.5,0.25,0.25,1], [-7,0,0.5,1], [80,2.01,0.33,1], [80,0.5,0.33,1]],
-                                2: [[0,0,0,0], [.25,1,.5,1], [1,1000,1,1], [.5,.2,.5,1], [1000,1,-3,1], 
+                                2: [[0,0,0,0], [.25,1,.5,1], [.25,.7,5000,1], [1,1000,1,1], [.5,.2,.5,1], [1000,1,-3,1], 
                                 [0,0,0,1], [-20,3,0,1], [-70,0.005,.01,1], [.7,.7,-12,1], [8,1,0,1], [100,5,1.1,1],
                                 [.1,0,0.5,1], [0.5,0.25,0.25,1], [-7,0,0.5,1], [80,2.01,0.33,1], [80,0.5,0.33,1]]}
-        self.pluginsDict = {'None': NonePlugin, 'Reverb': ReverbPlugin, 'Filter': FilterPlugin, 'Chorus': ChorusPlugin,
+        self.pluginsDict = {'None': NonePlugin, 'Reverb': ReverbPlugin, 'WGVerb': WGReverbPlugin, 'Filter': FilterPlugin, 'Chorus': ChorusPlugin,
                             'Para EQ': EQPlugin, '3 Bands EQ': EQ3BPlugin, 'Compress': CompressPlugin, 'Gate': GatePlugin,
                             'Disto': DistoPlugin, 'AmpMod': AmpModPlugin, 'Phaser': PhaserPlugin, 'Delay': DelayPlugin,
                             'Flange': FlangePlugin, 'Harmonizer': HarmonizerPlugin, 'Resonators': ResonatorsPlugin,
@@ -299,150 +479,6 @@ class CECControl(scrolled.ScrolledPanel):
 
         self.plugins = [plugin1, plugin2, plugin3]
         self.pluginsPanel.SetSizer(self.pluginSizer)
-        
-        sizerMain.Add(self.pluginsPanel, 1, wx.EXPAND | wx.ALL, 0)
- 
-        self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        controlPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        inputPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        self.outputPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        outputTextPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        outToolbox.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        self.peakLabel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        self.durationSlider.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        self.gainSlider.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        self.vuMeter.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-        self.pluginsPanel.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
-
-        self.SetSizer(sizerMain)
-
-        self.SetAutoLayout(1)
-        self.SetupScrolling(scroll_x = False)
-
-        wx.CallAfter(self.updateOutputFormat)
-
-    def OnLooseFocus(self, event):
-        win = wx.FindWindowAtPointer()
-        if win != None:
-            win = win.GetTopLevelParent()
-            if win not in [CeciliaLib.getVar("mainFrame"), CeciliaLib.getVar("interface")]:
-                win.Raise()
-        event.Skip()
-
-    def createGrapherLines(self, plugin):
-        knobs = [plugin.knob1, plugin.knob2, plugin.knob3]
-        grapher = CeciliaLib.getVar("grapher")
-        choice = grapher.toolbar.getPopupChoice()
-        choice.extend([knob.getLongLabel() for knob in knobs])
-        grapher.toolbar.setPopupChoice(choice)
-        for knob in knobs:
-            func = '0 %f 1 %f' % (knob.GetValue(), knob.GetValue())
-            func = [float(v.replace('"', '')) for v in func.split()]
-            func = [[func[i*2] * CeciliaLib.getVar("totalTime"), func[i*2+1]] for i in range(len(func) / 2)]
-            mini = knob.getRange()[0]
-            maxi = knob.getRange()[1]
-            colour = chooseColourFromName('red')
-            label = knob.getLongLabel()
-            log = knob.getLog()
-            name = knob.getName()
-            grapher.plotter.createLine(func, (mini, maxi), colour, label, log, name, 8192, knob, '')
-            grapher.plotter.getData()[-1].setShow(0)
-            grapher.plotter.draw()
-
-    def removeGrapherLines(self, plugin):
-        knobs = [plugin.knob1, plugin.knob2, plugin.knob3]
-        tmp = [knob.getLongLabel() for knob in knobs]
-        names = [knob.getName() for knob in knobs]
-        grapher = CeciliaLib.getVar("grapher")
-        choice = grapher.toolbar.getPopupChoice()
-        for label in tmp:
-            if label in choice:
-                choice.remove(label)
-        grapher.toolbar.setPopupChoice(choice)
-        grapher.plotter.removeLines(names)
-
-    def replacePlugin(self, order, new):
-        self.pluginsParams[order][self.oldPlugins[order]] = self.plugins[order].getParams()
-        oldPlugin = self.plugins[order]
-        if self.oldPlugins[order] != 0:
-            self.removeGrapherLines(oldPlugin)
-        plugin = self.pluginsDict[new](self.pluginsPanel, self.replacePlugin, order)
-
-        if new != 'None':    
-            CeciliaLib.setPlugins(plugin, order)
-            self.createGrapherLines(plugin)
-        else:
-            CeciliaLib.setPlugins(None, order)
-
-        ind = PLUGINS_CHOICE.index(plugin.getName())
-        self.oldPlugins[order] = ind
-        plugin.setParams(self.pluginsParams[order][ind])
-        if CeciliaLib.getVar("systemPlatform")  == 'darwin':
-            self.pluginSizer.Replace(oldPlugin, plugin)
-        else:
-            item = self.pluginSizer.GetItem(oldPlugin)
-            item.DeleteWindows()
-            self.pluginSizer.Insert([2,8,13][order], plugin, 0) 
-        self.plugins[order] = plugin       
-        self.pluginsPanel.Layout()
-
-    def setPlugins(self, pluginsDict):
-        for key in pluginsDict.keys():
-            self.replacePlugin(key, pluginsDict[key][0])
-            self.plugins[key].setParams(pluginsDict[key][1])
-            self.plugins[key].setStates(pluginsDict[key][2])
-            
-    def updateTime(self, time):
-        self.setTime(time)
-        self.GetParent().grapher.plotter.drawCursor(time)
-
-    def updateAmps(self, amps):
-        self.vuMeter.setAmplitude(amps)
-
-    def createInputPanel(self, parent):
-        isEmpty = True
-        inputPanel = wx.Panel(parent, -1, style=wx.NO_BORDER)
-        inputSizer = wx.FlexGridSizer(5,1)
-        
-        self.cfileinList = []
-        samplersList = []
-        widgets = CeciliaLib.getVar("interfaceWidgets")
-        
-        for w in range(len(widgets)):
-            if widgets[w]['type'] == 'cfilein':
-                cFileIn = Cfilein(inputPanel, label=widgets[w].get('label', ''), name=widgets[w]['name'])
-                self.cfileinList.append(cFileIn)
-            elif widgets[w]['type'] == 'csampler':
-                cSampler = CSampler(inputPanel, label=widgets[w].get('label', ''), name=widgets[w]['name'])
-                self.cfileinList.append(cSampler)
-                samplersList.append(cSampler)
-
-        CeciliaLib.setVar("userSamplers", samplersList)
-
-        if self.cfileinList != []:
-            isEmpty = False
-            # Section title
-            inputTextPanel = wx.Panel(inputPanel, -1, style=wx.NO_BORDER)
-            inputTextPanel.SetBackgroundColour(TITLE_BACK_COLOUR)
-            inputTextSizer = wx.FlexGridSizer(1,1)
-            inputText = wx.StaticText(inputTextPanel, -1, 'INPUT ')
-            inputText.SetFont(wx.Font(SECTION_TITLE_FONT, wx.NORMAL, wx.NORMAL, wx.BOLD, face=FONT_FACE))
-            inputText.SetBackgroundColour(TITLE_BACK_COLOUR)
-            inputText.SetForegroundColour(SECTION_TITLE_COLOUR)
-            inputTextSizer.Add(inputText, 0, wx.ALIGN_RIGHT | wx.ALL, 3)
-            inputTextSizer.AddGrowableCol(0)
-            inputTextPanel.SetSizer(inputTextSizer)
-            inputSizer.Add(inputTextPanel, 1, wx.EXPAND| wx.ALIGN_RIGHT | wx.ALL, 0)
-        
-        for i in range(len(self.cfileinList)):
-            inputSizer.Add(self.cfileinList[i], 1, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, -1)
-            if i != len(self.cfileinList)-1:
-                inputSizer.Add(Separator(inputPanel, size=(230,1)), 1, wx.EXPAND)
-        
-        inputSizer.AddGrowableCol(0)
-        inputPanel.SetSizer(inputSizer)
-        
-        return inputPanel, isEmpty
 
     def getCfileinList(self):
         return self.cfileinList
@@ -480,12 +516,13 @@ class CECControl(scrolled.ScrolledPanel):
     def onPlayStop(self, value):
         if value:
             self.nonZeroTime = 0
-            CeciliaLib.setVar("outputFile", 'dac')
+            CeciliaLib.setVar("toDac", True)
             CeciliaLib.getVar("grapher").toolbar.loadingMsg.SetForegroundColour("#FFFFFF")
+            CeciliaLib.getVar("grapher").toolbar.loadingMsg.Refresh()
             wx.CallLater(50, CeciliaLib.startCeciliaSound, True)
         else:
             CeciliaLib.stopCeciliaSound()
-        
+
     def onRec(self, value):
         if value:
             if self.outputFilename != '':
@@ -498,27 +535,68 @@ class CECControl(scrolled.ScrolledPanel):
                     return    
             self.outputFilename = filename
             CeciliaLib.setVar("outputFile", filename)
-            CeciliaLib.startCeciliaSound(timer=False)
-            self.updatePeak(0)
+            self.nonZeroTime = 0
+            CeciliaLib.setVar("toDac", True)
+            CeciliaLib.getVar("grapher").toolbar.loadingMsg.SetForegroundColour("#FFFFFF")
+            CeciliaLib.getVar("grapher").toolbar.loadingMsg.Refresh()
+            wx.CallLater(50, CeciliaLib.startCeciliaSound, True, True)
         else:
             CeciliaLib.stopCeciliaSound()
+
+    def onBounceToDisk(self):
+        if self.outputFilename != '':
+            filename = CeciliaLib.autoRename(self.outputFilename)
+            self.filenameLabel.setLabel(CeciliaLib.shortenName(os.path.split(filename)[1],self.charNumForLabel))
+        if self.outputFilename == '':
+            filename = self.onSelectOutputFilename()
+            if filename == None:
+                CeciliaLib.stopCeciliaSound()
+                return    
+        self.outputFilename = filename
+        CeciliaLib.setVar("outputFile", filename)
+        CeciliaLib.setVar("toDac", False)
+        self.showBounceDialog()
+        CeciliaLib.startCeciliaSound(timer=False)
+        self.updatePeak(0)
+
+    def showBounceDialog(self):
+        self.bounce_dlg = wx.Dialog(CeciliaLib.getVar('interface'), -1, "Bounce to disk")
+        sizer = wx.BoxSizer(wx.VERTICAL)
+        self.bounce_label = wx.StaticText(self.bounce_dlg, -1, "Writing %s on disk..." % self.outputFilename)
+        sizer.Add(self.bounce_label, 0, wx.ALIGN_CENTRE|wx.ALL, 25)
+        self.bounce_dlg.SetSizerAndFit(sizer)
+        self.bounce_dlg.CenterOnParent()
+        self.bounce_dlg.Show()
     
+    def closeBounceToDiskDialog(self):
+        try:
+            self.bounce_dlg.Destroy()
+        except:
+            pass
+
+    def onBatchProcessing(self, filename):
+        self.outputFilename = filename
+        CeciliaLib.setVar("outputFile", filename)
+        CeciliaLib.setVar("toDac", False)
+        CeciliaLib.startCeciliaSound(timer=False)
+        self.updatePeak(0)
+
     def onSelectOutputFilename(self):
         if CeciliaLib.getVar("audioFileType") == 'wav':
             wildcard = "Wave file|*.wave;*.WAV;*.WAVE;*.Wav;*.Wave*.wav|" \
                        "All files|*.*"
-        elif CeciliaLib.getVar("audioFileType") == 'aiff':
+        else:
             wildcard = "AIFF file|*.aiff;*.aifc;*.AIF;*.AIFF;*.Aif;*.Aiff*.aif|" \
                        "All files|*.*"
-        
+
         file = CeciliaLib.saveFileDialog(self, wildcard, type='Save audio')
-        
+
         if file != None:
             self.filenameLabel.setLabel(CeciliaLib.shortenName(os.path.split(file)[1],self.charNumForLabel))
             self.outputFilename = file
-            
-        return file    
-    
+
+        return file
+
     def updateOutputFormat(self):
         self.vuMeter.updateNchnls()
         x, y = self.meterSizer.GetPosition()
@@ -529,31 +607,35 @@ class CECControl(scrolled.ScrolledPanel):
         self.lineSizer.SetDimension(7, y+h+10, w2, h2)
         self.Layout()
         self.Refresh()
-            
+
     def onFormatChange(self, idx, choice):
         nchnls = int(choice)
         CeciliaLib.setVar("nchnls", nchnls)
         self.updateOutputFormat()
-        
+
     def onReuseOutputFile(self):
         if os.path.isfile(self.outputFilename):
             if self.cfileinList != []:
                 self.cfileinList[0].updateMenuFromPath(self.outputFilename)
-    
-    def setTotalTime(self, time):
-        if self.cfileinList != [] and time == 0:
-            time = self.cfileinList[0].getDuration()
-            self.durationSlider.SetValue(time)
-        CeciliaLib.setVar("totalTime", time)
-        if CeciliaLib.getVar("grapher"):
-            CeciliaLib.getVar("grapher").setTotalTime(time)
-    
+
+    def setTotalTime(self, time, force=False):
+        if CeciliaLib.getVar("audioServer").isAudioServerRunning() and not force:
+            self.tmpTotalTime = time
+        else:
+            if self.cfileinList != [] and time == 0:
+                time = self.cfileinList[0].getDuration()
+                self.durationSlider.SetValue(time)
+            CeciliaLib.setVar("totalTime", time)
+            if CeciliaLib.getVar("grapher"):
+                CeciliaLib.getVar("grapher").setTotalTime(time)
+            self.tmpTotalTime = time
+
     def updateDurationSlider(self):
         self.durationSlider.SetValue(CeciliaLib.getVar("totalTime"))
-    
+
     def updateNchnls(self):
         nchnls = CeciliaLib.getVar("nchnls")
-        
+
         if nchnls==1:
             format = 'Mono'
         elif nchnls==2:
@@ -566,13 +648,13 @@ class CECControl(scrolled.ScrolledPanel):
             format = 'Octo'
         else:
             format = 'Custom...'
-        
+
         self.formatChoice.setStringSelection(format)
         self.updateOutputFormat()
-        
+
     def onChangeGain(self, gain):
         CeciliaLib.getVar("audioServer").setAmp(gain)
-        
+
     def updatePeak(self, peak):
         self.peak = peak * 90.0 - 90.0
         label = ''
@@ -624,10 +706,10 @@ class Cfilein(wx.Panel):
         self.fileMenu = FolderPopup(self, path=None, init='', outFunction=self.onSelectSound,
                                     emptyFunction=self.onLoadFile, backColour=CONTROLLABEL_BACK_COLOUR, tooltip=TT_SEL_SOUND)
                                    
-        line2.Add(self.fileMenu, 0, wx.ALIGN_CENTER | wx.TOP | wx.RIGHT, 1)
+        line2.Add(self.fileMenu, 0, wx.ALIGN_CENTER | wx.RIGHT, 20)
         line2.AddSpacer((8,5))
-        self.toolbox = ToolBox(self, tools=['play','edit','load','open'],
-                               outFunction=[self.listenSoundfile,self.editSoundfile, self.onLoadFile, self.onShowSampler])
+        self.toolbox = ToolBox(self, tools=['play','edit','open'],
+                               outFunction=[self.listenSoundfile,self.editSoundfile, self.onShowSampler])
         line2.Add(self.toolbox, 0, wx.ALIGN_CENTER | wx.LEFT, 2)
         
         mainSizer.Add(line2, 1, wx.LEFT, 6)
@@ -700,9 +782,25 @@ class Cfilein(wx.Panel):
             return
         else:
             path = filePath
+        
+        if not CeciliaLib.getVar("audioServer").validateAudioFile(path):
+            CeciliaLib.showErrorDialog("Unable to retrieve sound infos", "There is something wrong with this file, please select another one.")
+            return
 
         if path:
             self.updateMenuFromPath(path)
+            lastfiles = CeciliaLib.getVar("lastAudioFiles")
+            if lastfiles != "":
+                lastfiles = lastfiles.split(";")
+            else:
+                lastfiles = []
+            if path in lastfiles:
+                return
+            if len(lastfiles) >= 10:
+                lastfile = lastfiles[1:]
+            lastfiles.append(path)
+            lastfiles = ";".join(lastfiles)
+            CeciliaLib.setVar("lastAudioFiles", lastfiles)
  
     def reset(self):
         self.fileMenu.reset()
@@ -710,7 +808,13 @@ class Cfilein(wx.Panel):
         CeciliaLib.getVar("userInputs")[self.name]['path'] = self.filePath
 
     def updateMenuFromPath(self, path):
-        root = os.path.split(path)[0]
+        if os.path.isfile(path):
+            root = os.path.split(path)[0]
+            isfile = True
+        elif os.path.isdir(path):
+            root = path
+            isfile = False
+        
         pathList = []
         for p in os.listdir(root):
             pathList.append(os.path.join(root,p))
@@ -719,8 +823,11 @@ class Cfilein(wx.Panel):
         files.sort()
         
         self.fileMenu.setChoice(files)
-        self.fileMenu.setLabel(CeciliaLib.ensureNFD(os.path.split(path)[1]))
-
+        if isfile:
+            self.fileMenu.setLabel(CeciliaLib.ensureNFD(os.path.split(path)[1]))
+        else:
+            self.fileMenu.setLabel(CeciliaLib.ensureNFD(files[0]))
+            
     def listenSoundfile(self):
         CeciliaLib.listenSoundfile(self.filePath)
     
@@ -795,11 +902,11 @@ class CSampler(Cfilein):
         line2 = wx.BoxSizer(wx.HORIZONTAL)
         self.fileMenu = FolderPopup(self, path=None, init='', outFunction=self.onSelectSound,
                                     emptyFunction=self.onLoadFile, backColour=CONTROLLABEL_BACK_COLOUR, tooltip=TT_SEL_SOUND)
-        line2.Add(self.fileMenu, 0, wx.ALIGN_CENTER | wx.TOP | wx.RIGHT, 1)
+        line2.Add(self.fileMenu, 0, wx.ALIGN_CENTER | wx.RIGHT, 20)
         line2.AddSpacer((8,5))
 
-        self.toolbox = ToolBox(self, tools=['play','edit','load','open'],
-                               outFunction=[self.listenSoundfile,self.editSoundfile, self.onLoadFile, self.onShowSampler],
+        self.toolbox = ToolBox(self, tools=['play','edit','open'],
+                               outFunction=[self.listenSoundfile,self.editSoundfile, self.onShowSampler],
                                openSampler=True)
         self.toolbox.setOpen(False)
         line2.Add(self.toolbox,0,wx.ALIGN_CENTER | wx.TOP | wx.LEFT, 2)
@@ -924,11 +1031,13 @@ class CfileinFrame(wx.Frame):
         textLabel2 = wx.StaticText(self, -1, self.parent.label)
         textLabel2.SetFont(wx.Font(TEXT_LABELFORWIDGET_FONT, wx.NORMAL, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         textLabel2.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
+        textLabel2.SetBackgroundColour(BACKGROUND_COLOUR)
         line3.Add(textLabel2,0,wx.ALL, 0)
         
         textOffset = wx.StaticText(self, -1, ' Offset :')
         textOffset.SetFont(wx.Font(TEXT_LABELFORWIDGET_FONT, wx.NORMAL, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         textOffset.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
+        textOffset.SetBackgroundColour(BACKGROUND_COLOUR)
         line3.Add(textOffset,0,wx.ALL, 0)
         
         box.Add(line3, 0, wx.LEFT, 20)
@@ -1015,11 +1124,13 @@ class SamplerFrame(wx.Frame):
         textLabel2 = wx.StaticText(panel, -1, self.parent.label)
         textLabel2.SetFont(wx.Font(TEXT_LABELFORWIDGET_FONT, wx.NORMAL, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         textLabel2.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
+        textLabel2.SetBackgroundColour(BACKGROUND_COLOUR)
         line3.Add(textLabel2,0,wx.ALL, 0)
         
         textOffset = wx.StaticText(panel, -1, ' Offset :')
         textOffset.SetFont(wx.Font(TEXT_LABELFORWIDGET_FONT, wx.NORMAL, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         textOffset.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
+        textOffset.SetBackgroundColour(BACKGROUND_COLOUR)
         line3.Add(textOffset,0,wx.ALL, 0)
         
         box.Add(line3, 0, wx.LEFT, 20)
@@ -1261,10 +1372,12 @@ class SamplerPlayRecButtons(wx.Panel):
         self.Bind(wx.EVT_PAINT, self.OnPaint)
         self.Bind(wx.EVT_MOTION, self.OnMotion)
         self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLeave)
+        self.Bind(wx.EVT_ENTER_WINDOW, self.OnEnter)
         self.Bind(wx.EVT_LEFT_DOWN, self.MouseDown)
         self.Bind(wx.EVT_LEFT_UP, self.MouseUp)
         self.playColour = SLIDER_PLAY_COLOUR_HOT
         self.recColour = SLIDER_REC_COLOUR_HOT
+        self.enterWithButtonDown = False
         self.playOver = False
         self.recOver = False
         self.playOverWait = True
@@ -1322,26 +1435,45 @@ class SamplerPlayRecButtons(wx.Panel):
         self.playOver = False
         self.recOver = False
         self.Refresh()
-        self.CaptureMouse()
         evt.Skip()
 
+    def OnEnter(self, evt):
+        if evt.ButtonIsDown(wx.MOUSE_BTN_LEFT) and not CeciliaLib.getVar("audioServer").isAudioServerRunning():
+            self.enterWithButtonDown = True
+            pos = evt.GetPosition()
+            if wx.Rect(0, 0, 20, 20).Contains(pos):
+                if self.play: 
+                    self.setPlay(0)
+                else:
+                    self.setPlay(1)
+            elif wx.Rect(20, 0, 40, 20).Contains(pos):
+                if self.rec: 
+                    self.setRec(False)
+                else: 
+                    self.setRec(True)
+            self.playOver = False
+            self.recOver = False
+            self.Refresh()
+            evt.Skip()
+
     def MouseUp(self, evt):
-        if self.HasCapture():
-            self.ReleaseMouse()
+        self.enterWithButtonDown = False
 
     def OnMotion(self, evt):
-        pos = evt.GetPosition()
-        if wx.Rect(2, 2, 17, 17).Contains(pos) and self.playOverWait:
-            self.playOver = True
-            self.recOver = False
-        elif wx.Rect(21, 2, 38, 17).Contains(pos) and self.recOverWait:
-            self.playOver = False
-            self.recOver = True
-        self.checkForOverReady(pos)
-        self.Refresh()
-        evt.Skip()
+        if not CeciliaLib.getVar("audioServer").isAudioServerRunning() and not self.enterWithButtonDown:
+            pos = evt.GetPosition()
+            if wx.Rect(2, 2, 17, 17).Contains(pos) and self.playOverWait:
+                self.playOver = True
+                self.recOver = False
+            elif wx.Rect(21, 2, 38, 17).Contains(pos) and self.recOverWait:
+                self.playOver = False
+                self.recOver = True
+            self.checkForOverReady(pos)
+            self.Refresh()
+            evt.Skip()
 
     def OnLeave(self, evt):
+        self.enterWithButtonDown = False
         self.playOver = False
         self.recOver = False
         self.playOverWait = True
