@@ -199,9 +199,13 @@ class PluginKnob(ControlKnob):
             if rec.Contains(pos):
                 if evt.ShiftDown():
                     self.setMidiCtl(None)
-                else:    
-                    CeciliaLib.getVar("audioServer").midiLearn(self)
-                    self.inMidiLearnMode()
+                else:
+                    if CeciliaLib.getVar("useMidi"):
+                        CeciliaLib.getVar("audioServer").midiLearn(self)
+                        self.inMidiLearnMode()
+                    else:
+                        CeciliaLib.showErrorDialog("Midi not initialized!",
+                            "There is no Midi interface connected!")
         evt.Skip()
 
 class Plugin(wx.Panel):
@@ -222,21 +226,21 @@ class Plugin(wx.Panel):
 
     def getKnobNames(self):
         return [knob.getName() for knob in self.getKnobs()]
-        
+
     def getKnobLongLabels(self):
         return [knob.getLongLabel() for knob in self.getKnobs()]
-        
+
     def getParams(self):
         return [self.knob1.GetValue(),self.knob2.GetValue(),self.knob3.GetValue(),self.preset.getIndex()]
 
     def getStates(self):
         return [self.knob1.getState(),self.knob2.getState(),self.knob3.getState()]
-        
+
     def setStates(self, states):
         self.knob1.setState(states[0])
         self.knob2.setState(states[1])
         self.knob3.setState(states[2])
-        
+
     def setParams(self, params):
         self.knob1.SetValue(params[0])
         self.knob2.SetValue(params[1])
@@ -248,18 +252,21 @@ class Plugin(wx.Panel):
         self.onChangeKnob2(self.knob2.GetValue())
         self.onChangeKnob3(self.knob3.GetValue())
         self.onChangePreset(self.preset.getIndex())
-        
+
     def onChangeKnob1(self, x):
-        if CeciliaLib.getVar("currentModule") != None:
-            CeciliaLib.getVar("audioServer").setPluginValue(self.order, 0, x)
+        if self.knob1.getState()[1] in [0,1]:
+            if CeciliaLib.getVar("currentModule") != None:
+                CeciliaLib.getVar("audioServer").setPluginValue(self.order, 0, x)
 
     def onChangeKnob2(self, x):
-        if CeciliaLib.getVar("currentModule") != None:
-            CeciliaLib.getVar("audioServer").setPluginValue(self.order, 1, x)
+        if self.knob2.getState()[1] in [0,1]:
+            if CeciliaLib.getVar("currentModule") != None:
+                CeciliaLib.getVar("audioServer").setPluginValue(self.order, 1, x)
 
     def onChangeKnob3(self, x):
-        if CeciliaLib.getVar("currentModule") != None:
-            CeciliaLib.getVar("audioServer").setPluginValue(self.order, 2, x)
+        if self.knob3.getState()[1] in [0,1]:
+            if CeciliaLib.getVar("currentModule") != None:
+                CeciliaLib.getVar("audioServer").setPluginValue(self.order, 2, x)
 
     def onChangePreset(self, x, label=None):
         if CeciliaLib.getVar("currentModule") != None:
@@ -288,7 +295,7 @@ class NonePlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='None', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='None', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
         
@@ -329,7 +336,7 @@ class ReverbPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Reverb', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Reverb', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
         
@@ -344,6 +351,47 @@ class ReverbPlugin(Plugin):
         self.sizer.Add(revMenuBox, 0, wx.LEFT, 5)
         self.SetSizer(self.sizer)
 
+class WGReverbPlugin(Plugin):
+    def __init__(self, parent, choiceFunc, order):
+        Plugin.__init__(self, parent, choiceFunc, order)
+        self.pluginName = 'WGVerb'
+        self.sizer = wx.FlexGridSizer(1,4,0,0)
+        revMenuBox = wx.BoxSizer(wx.VERTICAL)
+
+        self.knob1 = PluginKnob(self, 0, 1, 0.25, size=(43,70), log=False, outFunction=self.onChangeKnob1, label='Mix')
+        self.knob1.setName('plugin_%d_wgreverb_mix' % self.order)       
+        self.knob1.setLongLabel('plugin %d WGVerb Mix' % (self.order+1))       
+        self.sizer.Add(self.knob1)
+
+        self.knob2 = PluginKnob(self, 0., 1, 0.7, size=(43,70), log=False, outFunction=self.onChangeKnob2, label='Feed')        
+        self.knob2.setName('plugin_%d_wgreverb_feed' % self.order)       
+        self.knob2.setLongLabel('plugin %d WGVerb Feed' % (self.order+1))       
+        self.sizer.Add(self.knob2)
+
+        self.knob3 = PluginKnob(self, 100, 15000, 5000, size=(43,70), log=True, outFunction=self.onChangeKnob3, label='Cutoff')        
+        self.knob3.setName('plugin_%d_wgreverb_lp' % self.order)       
+        self.knob3.setLongLabel('plugin %d WGVerb Cutoff' % (self.order+1))       
+        self.sizer.Add(self.knob3)
+
+        plugChoiceText = wx.StaticText(self, -1, 'Effects:')
+        plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
+        plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
+        revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='WGVerb', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
+        self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
+        revMenuBox.Add(self.choice, 0, wx.TOP, 2)
+
+        plugChoicePreset = wx.StaticText(self, -1, 'Type:')
+        plugChoicePreset.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
+        plugChoicePreset.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
+        revMenuBox.Add(plugChoicePreset, 0, wx.TOP, 6)
+        self.preset = CustomMenu(self, choice=['Bypass', 'Active'], init='Active', size=(93,18), colour=CONTROLLABEL_BACK_COLOUR, outFunction=self.onChangePreset)
+        self.presetName = 'plugin_%d_wgreverb_preset' % self.order                     
+        revMenuBox.Add(self.preset, 0, wx.TOP, 2)
+
+        self.sizer.Add(revMenuBox, 0, wx.LEFT, 5)
+        self.SetSizer(self.sizer)
+
 class FilterPlugin(Plugin):
     def __init__(self, parent, choiceFunc, order):
         Plugin.__init__(self, parent, choiceFunc, order)
@@ -371,7 +419,7 @@ class FilterPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Filter', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Filter', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -414,7 +462,7 @@ class EQPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Para EQ', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Para EQ', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -459,7 +507,7 @@ class EQ3BPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='3 Bands EQ', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='3 Bands EQ', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -501,7 +549,7 @@ class ChorusPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Chorus', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Chorus', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -545,7 +593,7 @@ class CompressPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Compress', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Compress', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -587,7 +635,7 @@ class GatePlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Gate', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Gate', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -629,7 +677,7 @@ class DistoPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Disto', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Disto', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -671,7 +719,7 @@ class AmpModPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='AmpMod', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='AmpMod', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -714,7 +762,7 @@ class PhaserPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Phaser', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Phaser', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -756,7 +804,7 @@ class DelayPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Delay', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Delay', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -798,7 +846,7 @@ class FlangePlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Flange', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Flange', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -840,7 +888,7 @@ class HarmonizerPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Harmonizer', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Harmonizer', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -884,7 +932,7 @@ class ResonatorsPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Resonators', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='Resonators', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
@@ -927,7 +975,7 @@ class DeadResonPlugin(Plugin):
         plugChoiceText.SetFont(wx.Font(CONTROLSLIDER_FONT, wx.ROMAN, wx.NORMAL, wx.NORMAL, face=FONT_FACE))
         plugChoiceText.SetForegroundColour(TEXT_LABELFORWIDGET_COLOUR)
         revMenuBox.Add(plugChoiceText, 0, wx.TOP, 0)
-        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='DeadReson', size=(93,18), colour=CPOLY_COLOUR, outFunction=self.replacePlugin)
+        self.choice = CustomMenu(self, choice=PLUGINS_CHOICE, init='DeadReson', size=(93,18), colour=PLUGINPOPUP_BACK_COLOUR, outFunction=self.replacePlugin)
         self.choice.SetToolTip(CECTooltip(TT_POST_ITEMS))
         revMenuBox.Add(self.choice, 0, wx.TOP, 2)
 
