@@ -85,7 +85,7 @@ class PreferenceFrame(wx.Frame):
         closerBox = wx.BoxSizer(wx.HORIZONTAL)
         closer = CloseBox(panel, outFunction=self.onClose)
         closerBox.Add(closer, 0, wx.LEFT, 290)
-        box.Add(closerBox, 0, wx.TOP, 80)
+        box.Add(closerBox, 0, wx.TOP, 35)
         box.AddSpacer(10)
 
         panel.SetSizerAndFit(box)
@@ -131,6 +131,16 @@ class PreferenceFrame(wx.Frame):
         self.textSfEditorPath.SetBackgroundColour("#999999")
         buttonSfEditorPath = CloseBox(pathsPanel, outFunction=self.changeSfEditor, label='Set...')
 
+        #Text Editor
+        textTxtEditorLabel = wx.StaticText(pathsPanel, -1, 'Text Editor :')
+        textTxtEditorLabel.SetFont(self.font)       
+        self.textTxtEditorPath = wx.TextCtrl(pathsPanel, -1, CeciliaLib.getVar("textEditor"), size=(274,16), style=wx.TE_PROCESS_ENTER|wx.NO_BORDER)
+        self.textTxtEditorPath.SetFont(self.font)       
+        self.textTxtEditorPath.Bind(wx.EVT_TEXT_ENTER, self.handleEditTextEditorPath)
+        self.textTxtEditorPath.SetForegroundColour((50,50,50))
+        self.textTxtEditorPath.SetBackgroundColour("#999999")
+        buttonTxtEditorPath = CloseBox(pathsPanel, outFunction=self.changeTxtEditor, label='Set...')
+
         textPrefPathLabel = wx.StaticText(pathsPanel, -1, 'Preferred paths :')
         textPrefPathLabel.SetFont(self.font)       
         self.textPrefPath = wx.TextCtrl(pathsPanel, -1, CeciliaLib.getVar("prefferedPath"), size=(274,16), style=wx.TE_PROCESS_ENTER|wx.NO_BORDER)
@@ -149,6 +159,10 @@ class PreferenceFrame(wx.Frame):
                             (wx.StaticText(pathsPanel, -1, ''), 0, wx.EXPAND | wx.LEFT, 5),
                             (self.textSfEditorPath, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, PADDING),
                             (buttonSfEditorPath, 0, wx.RIGHT, 10),
+                            (textTxtEditorLabel, 0, wx.EXPAND | wx.LEFT | wx.TOP, PADDING),
+                            (wx.StaticText(pathsPanel, -1, ''), 0, wx.EXPAND | wx.LEFT, 5),
+                            (self.textTxtEditorPath, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, PADDING),
+                            (buttonTxtEditorPath, 0, wx.RIGHT, 10),
                             (textPrefPathLabel, 0, wx.EXPAND | wx.LEFT | wx.TOP, PADDING),
                             (wx.StaticText(pathsPanel, -1, ''), 0, wx.EXPAND | wx.LEFT, 5),
                             (self.textPrefPath, 0,  wx.LEFT | wx.ALIGN_CENTER_VERTICAL, PADDING),
@@ -186,6 +200,10 @@ class PreferenceFrame(wx.Frame):
         textgraphTexture.SetFont(self.font)       
         self.textureToggle = Toggle(ceciliaPanel, CeciliaLib.getVar("graphTexture"), outFunction=self.enableGraphTexture)
 
+        textVerbose = wx.StaticText(ceciliaPanel, 0, 'Verbose :')
+        textVerbose.SetFont(self.font)       
+        self.verboseToggle = Toggle(ceciliaPanel, CeciliaLib.getVar("DEBUG"), outFunction=self.enableVerbose)
+
         if sys.platform == "linux2":
             spacerSize = 70
         else:
@@ -203,6 +221,9 @@ class PreferenceFrame(wx.Frame):
                             (textgraphTexture, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, PADDING),
                             (wx.StaticText(ceciliaPanel, -1, ''), 1, wx.EXPAND),
                             (self.textureToggle, 0, wx.ALIGN_CENTER_VERTICAL),
+                            (textVerbose, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, PADDING),
+                            (wx.StaticText(ceciliaPanel, -1, ''), 1, wx.EXPAND),
+                            (self.verboseToggle, 0, wx.ALIGN_CENTER_VERTICAL),
                          ])
 
         gridSizer.AddGrowableCol(1, 1)
@@ -268,10 +289,21 @@ class PreferenceFrame(wx.Frame):
         self.midiDriverPanels.append(portmidiPane)
         self.midiDriverBox.Add(self.midiDriverPanels[self.midiDriverCurrentPane])
 
+        gridSizer2 = wx.FlexGridSizer(5, 3, 5, 5)
+        textAutoBinding = wx.StaticText(midiParamPanel, 0, 'Automatic Midi Bindings :')
+        textAutoBinding.SetFont(self.font)       
+        self.autoMidiToggle = Toggle(midiParamPanel, CeciliaLib.getVar("automaticMidiBinding"), outFunction=self.enableAutomaticBinding)
+        gridSizer2.AddMany([ 
+                            (textAutoBinding, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, PADDING),
+                            (self.autoMidiToggle, 1, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 170),
+                            ])
+
         gridSizer1.AddGrowableCol(1, 1)
         box.Add(gridSizer1)
         box.AddSpacer(20)
         box.Add(self.midiDriverBox)
+        box.AddSpacer(20)
+        box.Add(gridSizer2)
         midiParamPanel.SetSizerAndFit(box)
         return midiParamPanel
 
@@ -381,7 +413,7 @@ class PreferenceFrame(wx.Frame):
         for d in CeciliaLib.getVar("availableMidiInputs"):
             availableMidiIns.append(CeciliaLib.ensureNFD(d))
         try:
-            initInput = availableMidiIns[int(CeciliaLib.getVar("midiDeviceIn"))]
+            initInput = availableMidiIns[CeciliaLib.getVar("availableMidiInputIndexes").index(CeciliaLib.getVar("midiDeviceIn"))]
         except:
             if len(availableMidiIns) >= 1:
                 initInput = availableMidiIns[0]
@@ -412,7 +444,7 @@ class PreferenceFrame(wx.Frame):
         for d in CeciliaLib.getVar("availableAudioInputs"):
             availableAudioIns.append(CeciliaLib.ensureNFD(d))
         try:
-            initInput = availableAudioIns[CeciliaLib.getVar("audioInput")]
+            initInput = availableAudioIns[CeciliaLib.getVar("availableAudioInputIndexes").index(CeciliaLib.getVar("audioInput"))]
         except:
             if len(availableAudioIns) >= 1:
                 initInput = availableAudioIns[0]
@@ -433,9 +465,12 @@ class PreferenceFrame(wx.Frame):
         for d in CeciliaLib.getVar("availableAudioOutputs"):
             availableAudioOuts.append(CeciliaLib.ensureNFD(d))
         try:
-            initOutput = availableAudioOuts[CeciliaLib.getVar("audioOutput")]
+            initOutput = availableAudioOuts[CeciliaLib.getVar("availableAudioOutputIndexes").index(CeciliaLib.getVar("audioOutput"))]
         except:
-            initOutput = availableAudioOuts[0]
+            if len(availableAudioOuts) >= 1:
+                initOutput = availableAudioOuts[0]
+            else:
+                initOutput = ''
         self.choiceOutput = CustomMenu(portaudioPanel, choice=availableAudioOuts, init=initOutput, 
                                        size=(168,20), outFunction=self.changeAudioOutput, maxChar=25)
         
@@ -507,61 +542,28 @@ class PreferenceFrame(wx.Frame):
             CeciliaLib.setVar("audioInput", 0)
 
     def changeAudioInput(self, index, label):
-        CeciliaLib.setVar("audioInput", index)
+        deviceIndex = CeciliaLib.getVar("availableAudioInputIndexes")[index]
+        CeciliaLib.setVar("audioInput", deviceIndex)
 
     def changeAudioOutput(self, index, label):
-        CeciliaLib.setVar("audioOutput", index)
+        deviceIndex = CeciliaLib.getVar("availableAudioOutputIndexes")[index]
+        CeciliaLib.setVar("audioOutput", deviceIndex)
 
     def changeMidiInput(self, index, label):
-        CeciliaLib.setVar("midiDeviceIn", index)
+        deviceIndex = CeciliaLib.getVar("availableMidiInputIndexes")[index]
+        CeciliaLib.setVar("midiDeviceIn", deviceIndex)
 
     def changeSfPlayer(self):
-        if CeciliaLib.getVar("systemPlatform")  == 'win32':
-            wildcard =  "Executable files (*.exe)|*.exe|"     \
-                        "All files (*.*)|*.*"
-        elif CeciliaLib.getVar("systemPlatform")  == 'darwin':
-            wildcard =  "Application files (*.app)|*.app|"     \
-                        "All files (*.*)|*.*"
-        else:
-            wildcard = "All files (*.*)|*.*"
-
-        path = ''
-        dlg = wx.FileDialog(self, message="Choose a soundfile player...",
-                                 defaultDir=os.path.expanduser('~'),
-                                 wildcard=wildcard,
-                                 style=wx.OPEN)
-
-        if dlg.ShowModal() == wx.ID_OK:
-            path = dlg.GetPath()   
-        dlg.Destroy()
-
-        if path:
-            CeciliaLib.setVar("soundfilePlayer", path)
-            self.textSfPlayerPath.SetValue(path)
+        CeciliaLib.loadPlayerEditor("soundfile player")
+        self.textSfPlayerPath.SetValue(CeciliaLib.getVar("soundfilePlayer"))
 
     def changeSfEditor(self):
-        if CeciliaLib.getVar("systemPlatform")  == 'win32':
-            wildcard =  "Executable files (*.exe)|*.exe|"     \
-                        "All files (*.*)|*.*"
-        elif CeciliaLib.getVar("systemPlatform")  == 'darwin':
-            wildcard =  "Application files (*.app)|*.app|"     \
-                        "All files (*.*)|*.*"
-        else:
-            wildcard = "All files (*.*)|*.*"
+        CeciliaLib.loadPlayerEditor("soundfile editor")
+        self.textSfEditorPath.SetValue(CeciliaLib.getVar("soundfileEditor"))
 
-        path = ''
-        dlg = wx.FileDialog(self, message="Choose a soundfile editor...",
-                                 defaultDir=os.path.expanduser('~'),
-                                 wildcard=wildcard,
-                                 style=wx.OPEN)
-
-        if dlg.ShowModal() == wx.ID_OK:
-            path = dlg.GetPath()   
-        dlg.Destroy() 
-
-        if path:
-            CeciliaLib.setVar("soundfileEditor", path)
-            self.textSfEditorPath.SetValue(path)
+    def changeTxtEditor(self):
+        CeciliaLib.loadPlayerEditor("text editor")
+        self.textTxtEditorPath.SetValue(CeciliaLib.getVar("textEditor"))
 
     def addPrefPath(self):
         currentPath = CeciliaLib.getVar("prefferedPath")
@@ -592,6 +594,11 @@ class PreferenceFrame(wx.Frame):
         CeciliaLib.setVar("soundfileEditor", path)
         self.textSfEditorPath.Navigate()
 
+    def handleEditTextEditorPath(self, event):
+        path = self.textTxtEditorPath.GetValue()
+        CeciliaLib.setVar("textEditor", path)
+        self.textTxtEditorPath.Navigate()
+
     def handleEditPrefPath(self, event):
         path = self.textPrefPath.GetValue()
         CeciliaLib.setVar("prefferedPath", path)
@@ -635,3 +642,9 @@ class PreferenceFrame(wx.Frame):
         CeciliaLib.setVar("graphTexture", state)
         if CeciliaLib.getVar("grapher") != None:
             CeciliaLib.getVar("grapher").plotter.draw()
+
+    def enableAutomaticBinding(self, state):
+        CeciliaLib.setVar("automaticMidiBinding", state)
+
+    def enableVerbose(self, state):
+        CeciliaLib.setVar("DEBUG", state)
