@@ -1,4 +1,12 @@
 rm -rf build dist
+
+export DMG_DIR="Cecilia 5.0.8"
+export DMG_NAME="Cecilia_5.0.8.dmg"
+
+if [ -f setup.py ]; then
+    mv setup.py setup_back.py;
+fi
+
 py2applet --make-setup Cecilia5.py Resources/*
 python setup.py py2app --plist=scripts/info.plist
 rm -f setup.py
@@ -30,6 +38,20 @@ cd Cecilia5.app/Contents
 awk '{gsub("Library/Frameworks/Python.framework/Versions/2.6/Resources/Python.app/Contents/MacOS/Python", "@executable_path/../Frameworks/Python.framework/Versions/2.6/Python")}1' Info.plist > Info.plist_tmp && mv Info.plist_tmp Info.plist
 
 cd ../..
-tar -cjvf Cecilia5_OSX-5.0.7.tar.bz2 Cecilia5.app
+echo "assembling DMG..."
+mkdir "$DMG_DIR"
+cd "$DMG_DIR"
+cp -R ../Cecilia5.app .
+ln -s /Applications .
+cd ..
+
+hdiutil create "$DMG_NAME" -srcfolder "$DMG_DIR"
+
+rm -rf "$DMG_DIR"
 rm -rf Cecilia5_OSX
 rm -rf Cecilia5.app
+
+if [ -f setup_back.py ]; then
+    mv setup_back.py setup.py;
+fi
+
